@@ -212,8 +212,8 @@ but pass in only 2 arrays (y, color).
 
 Gnuplot can plot both data and equations. This module exists largely for the
 data-plotting case, but sometimes it can be useful to plot equations together
-with some data. This is supported by the 'equation' plot option. This plot
-option is either a string (for a single equation) or a list/tuple containing
+with some data. This is supported by the 'equation...' plot options. These plot
+options are either a string (for a single equation) or a list/tuple containing
 multiple strings for multiple equations. An example:
 
     import numpy as np
@@ -390,13 +390,19 @@ gnuplotlib will flip the y axis to make things look reasonable. If any y-axis
 ranges are given, however (with any of the ymin,ymax,yrange,yinv plot options),
 then it is up to the user to flip the axis, if that's what they want.
 
-- equation
+- equation, equation_above, equation_below
 
-This option allows equations represented as formula strings to be plotted along
-with data passed in as numpy arrays. This can be a string (for a single
+These options allows equations represented as formula strings to be plotted
+along with data passed in as numpy arrays. These can be a string (for a single
 equation) or an array/tuple of strings (for multiple equations). See the
 "Symbolic equations" section above.
 
+By default, the equations are plotted BEFORE other data, so the data plotted
+later may obscure some of the equation. Depending on what we're doing, this may
+or may not be what we want. To plot the equations AFTER other data, use
+'equation_above' instead of 'equation'. The 'equation_below' option is a synonym
+for 'equation'
+
 - hardcopy
 
 Instead of drawing a plot on screen, plot into a file instead. The output
@@ -800,7 +806,7 @@ import numpy as np
 import numpysane as nps
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.26'
+__version__ = '0.27'
 
 # note that 'with' is both a known plot and curve option
 knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest', 'wait',
@@ -808,7 +814,8 @@ knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest', 'wait',
                               '3d',
                               'cmds', 'set', 'unset', 'square', 'square_xy', 'title',
                               'hardcopy', 'terminal', 'output',
-                              'with', 'equation', 'rgbimage',
+                              'with', 'rgbimage',
+                              'equation', 'equation_above', 'equation_below',
                               'xmax',  'xmin',  'xrange',  'xinv',  'xlabel',
                               'y2max', 'y2min', 'y2range', 'y2inv', 'y2label',
                               'ymax',  'ymin',  'yrange',  'yinv',  'ylabel',
@@ -1144,13 +1151,11 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
 
     def __del__(self):
         if hasattr(self, 'gnuplotProcess') and self.gnuplotProcess:
-            if self.checkpoint_stuck:
+
+            try:
                 self.gnuplotProcess.terminate()
-            else:
-                try:
-                    self._printGnuplotPipe( "exit\n" )
-                except:
-                    pass
+            except:
+                pass
 
             try:
                 self.gnuplotProcess.wait()
@@ -1361,13 +1366,14 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
         #    as a command. The plot command itself will never be invalid, so this
         #    doesn't actually mask out any errors
         #
-        # 2. "invalid range" errors caused by requested plot bounds (xmin, xmax,
-        #    etc) tossing out any test-plot data. The point of the plot-command
-        #    testing is to make sure the command is valid, so any out-of-boundedness
-        #    of the test data is irrelevant
+        # 2. "invalid range" and "Terminal canvas area too small to hold plot"
+        #    errors caused by the data or labels being out of bounds. The point
+        #    of the plot-command testing is to make sure the command is valid,
+        #    so any out-of-boundedness of the test data is irrelevant
         #
         # 3. image grid complaints
         if flags == 'ignore_known_test_failures':
+
             r = re.compile(r'''^gnuplot>\s*(?:{}|e\b).*$                  # report of the actual invalid command
                                \n^\s+\^\s*$                               # ^ mark pointing to where the error happened
                                \n^.*invalid\s+command.*$'''               # actual 'invalid command' complaint
@@ -1395,7 +1401,16 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
             # Newer gnuplot sometimes says 'x_min should not equal x_max!' when
             # complaining about ranges. Ignore those here
             r = re.compile(r'^.*_min should not equal .*_max!.*$',        # actual 'invalid range' complaint
-                           re.X + re.M)
+                           re.M)
+            fromerr = r.sub('', fromerr)
+
+            # Labels or titles that are too long can complain about stuff being
+            # too small to hold plot
+            r = re.compile(r'''^.*too small to hold plot.*$''',
+                           re.M)
+            fromerr = r.sub('', fromerr)
+            r = re.compile(r'''^.*Check plot boundary.*$''',
+                           re.M)
             fromerr = r.sub('', fromerr)
 
             # 'with image' plots can complain about an uninteresting domain. Exact error:
@@ -1574,23 +1589,28 @@ labels with spaces in them
         if self.plotOptions.get('3d'): basecmd += 'splot '
         else:                          basecmd += 'plot '
 
-        plotCurveCmdsNonData = []
-        plotCurveCmds        = []
-        plotCurveCmdsMinimal = [] # same as above, but with a single data point per plot only
+        plotCurveCmdsNonDataBefore = []
+        plotCurveCmdsNonDataAfter  = []
+        plotCurveCmds              = []
+        plotCurveCmdsMinimal       = [] # same as above, but with a single data point per plot only
 
-        # send all equations
-        if 'equation' in self.plotOptions:
-            if isinstance(self.plotOptions['equation'], (list, tuple)):
-                plotCurveCmdsNonData += self.plotOptions['equation']
-            else:
-                plotCurveCmdsNonData.append(self.plotOptions['equation'])
+        # send all pre-data equations
+        def set_equation(equation, cmds):
+            if equation in self.plotOptions:
+                if isinstance(self.plotOptions[equation], (list, tuple)):
+                    cmds += self.plotOptions[equation]
+                else:
+                    cmds.append(self.plotOptions[equation])
+
+        set_equation('equation',       plotCurveCmdsNonDataBefore)
+        set_equation('equation_below', plotCurveCmdsNonDataBefore)
 
         if 'rgbimage' in self.plotOptions:
             if not os.access     (self.plotOptions['rgbimage'], os.R_OK) or \
                not os.path.isfile(self.plotOptions['rgbimage']):
                 raise GnuplotlibError("Requested image '{}' is not a readable file".format(self.plotOptions['rgbimage']))
 
-            plotCurveCmdsNonData.append('"{0}" binary filetype=auto flipy with rgbimage title "{0}"'.format(self.plotOptions['rgbimage']))
+            plotCurveCmdsNonDataBefore.append('"{0}" binary filetype=auto flipy with rgbimage title "{0}"'.format(self.plotOptions['rgbimage']))
 
         testData             = '' # data to make a minimal plot
 
@@ -1646,9 +1666,11 @@ labels with spaces in them
 
                 testData += testData_curve
 
+        set_equation('equation_above', plotCurveCmdsNonDataAfter)
+
         # the command to make the plot and to test the plot
-        cmd        =  basecmd + ','.join(plotCurveCmdsNonData + plotCurveCmds)
-        cmdMinimal = (basecmd + ','.join(plotCurveCmdsNonData + plotCurveCmdsMinimal)) \
+        cmd        =  basecmd + ','.join(plotCurveCmdsNonDataBefore + plotCurveCmds        + plotCurveCmdsNonDataAfter)
+        cmdMinimal = (basecmd + ','.join(plotCurveCmdsNonDataBefore + plotCurveCmdsMinimal + plotCurveCmdsNonDataAfter)) \
             if plotCurveCmdsMinimal else cmd
 
         return (cmd, cmdMinimal, testData)
